from enum import Enum, auto

class DepEnum(Enum):
    ACL = auto()
    ADVCL = auto()
    ADVMOD = auto()
    AMOD = auto()
    APPOS = auto()
    AUX = auto()
    CASE = auto()
    CC = auto()
    CCOMP = auto()
    CLF = auto()
    COMPOUND = auto()
    CONJ = auto()
    COP = auto()
    CSUBJ = auto()
    DEP = auto()
    DET	 = auto()
    DISCOURSE = auto()
    DISLOCATED = auto()
    EXPL = auto()
    FIXED = auto()
    FLAT = auto()
    GOESWITH = auto()
    IOBJ = auto()
    LIST = auto()
    MARK = auto()
    NMOD = auto()
    NSUBJ = auto()
    NUMMOD = auto()
    OBJ = auto()
    OBL = auto()
    ORPHAN = auto()
    PARATAXIS = auto()
    PUNCT = auto()
    REPARANDUM = auto()
    ROOT = auto()
    VOCATIVE = auto()
    XCOMP = auto()
